#include <stdlib.h>
#include <stdio.h>
#include "canvas.h"
#include <GL/gl.h>
#include <GL/glx.h>

svg_canvas_t
svg_canvas_create(int width, int height) {
  svg_canvas_t canvas;
  XSetWindowAttributes attr;
  int attrib[] = { GLX_RGBA,
		   GLX_RED_SIZE, 1,
		   GLX_GREEN_SIZE, 1,
		   GLX_BLUE_SIZE, 1,
		   GLX_DOUBLEBUFFER,
		   None };
  unsigned long mask;

  canvas = malloc(sizeof(struct svg_canvas));
  if (!canvas) return NULL;

  canvas->dpy = XOpenDisplay(NULL);
  if (!canvas->dpy) {
    printf("can't connect X server\n");
    free(canvas);
    return NULL;
  }

  canvas->scrnum = DefaultScreen(canvas->dpy);
  canvas->root = RootWindow(canvas->dpy, canvas->scrnum);

  canvas->visinfo = glXChooseVisual(canvas->dpy, canvas->scrnum, attrib );
  if (!canvas->visinfo) {
    printf("can't get visual\n");
    free(canvas);
    return NULL;
  }
  
  /* window attributes */
  attr.background_pixel = 0;
  attr.border_pixel = 0;
  attr.colormap = XCreateColormap( canvas->dpy, canvas->root, canvas->visinfo->visual, AllocNone);
  attr.event_mask = StructureNotifyMask | ExposureMask;
  mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

  canvas->win = XCreateWindow(canvas->dpy, canvas->root, 0, 0, width, height,
			      0, canvas->visinfo->depth, InputOutput,
			      canvas->visinfo->visual, mask, &attr );

  canvas->width = width;
  canvas->height = height;

  {
    XSizeHints sizehints;
    sizehints.x = 0;
    sizehints.y = 0;
    sizehints.width  = width;
    sizehints.height = height;
    sizehints.flags = USSize | USPosition;
    XSetNormalHints(canvas->dpy, canvas->win, &sizehints);
    XSetStandardProperties(canvas->dpy, canvas->win, "blah", "BLAH",
			   None, (char **)NULL, 0, &sizehints);
  }

  canvas->glx_context = glXCreateContext(canvas->dpy, canvas->visinfo,
					 NULL, True );

  glXMakeCurrent(canvas->dpy, canvas->win, canvas->glx_context);

  XMapWindow(canvas->dpy, canvas->win);

  return canvas;
}

void
svg_canvas_draw(svg_canvas_t canvas) {
  glFlush();
  glXSwapBuffers(canvas->dpy, canvas->win);
}

void
svg_canvas_resize(svg_canvas_t canvas, int width, int height) {
  glViewport( 0, 0, width, height );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho( -1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );
}

void
svg_draw_element(dtdtc_element_t element) {
  switch (element->type) {
  case SVG_ELEMENT_TEXT:
    svg_text_draw( (svg_element_text_t)element );
    break;
  case SVG_ELEMENT_PATH:
    svg_path_draw( (svg_element_path_t)element );
    break;
  case SVG_ELEMENT_RECT:
    svg_rect_draw( (svg_element_rect_t)element );
    break;
  case SVG_ELEMENT_G:
    svg_g_draw( (svg_element_g_t)element );
    break;
  case SVG_ELEMENT_A:
    svg_a_draw( (svg_element_a_t)element );
    break;
  case SVG_ELEMENT_SVG:
    svg_svg_draw( (svg_element_svg_t)element );
    break;
  case SVG_ELEMENT_IMAGE:
    svg_image_draw( (svg_element_image_t)element );
    break;
  }
}
