#include "svg-parser.h"
#include "player.h"

#include <math.h>
#include <stdlib.h>


/* specific accessors */

/*
 * set length
 * return 0 on success, 1 otherwise
 * TODO: this is for 90dpi
 */
static int
svg_css_length_set(double *length, const xmlChar *value, double reference) {
  xmlChar *tail;

  *length = strtod(value, (char**)&tail) / reference;
  if (tail != value) {
    switch(*tail) {
    case '\0':
    case ' ':
      return 0;
    case '%':
      *length *= 0.01;
      return 0;
    case 'p':
      switch (*(tail + 1)) {
      case 't':
	*length *= 1.25;
	return 0;
      case 'c':
	*length *= 15;
	return 0;
      case 'x':
	return 0;
      }
      break;
    case 'c':
      if (*(tail + 1) == 'm') {
	*length *= 35.43307;
	return 0;
      }
      break;
    case 'm':
      if (*(tail + 1) == 'm') {
	*length *= 3.543307;
	return 0;
      }
      break;
    case 'i':
      if (*(tail + 1) == 'n') {
	*length *= 3.543307;
	return 0;
      }
      break;
    }
  }
  return 1;
}

void
svg_length_set(dtdtc_context_t ctx, double *attribute,
	       const xmlChar *value, void *data) {
  double *x_base;
  double *y_base;
  double reference;
  dtdtc_element_t parent;
  int result;

  parent = dtdtc_lookup_parent_by_attribute(ctx, ctx->element, "width");
  if (parent) {
    x_base = dtdtc_attribute_get(ctx, parent, "width");
    y_base = dtdtc_attribute_get(ctx, parent, "height");
  
    if (y_base) {
      reference = sqrt(*x_base * *x_base + *y_base * *y_base) / sqrt(2);
    } else {
      reference = 1.0;
    }
  } else {
    reference = 1.0;
  }
  result = svg_css_length_set(attribute, value, reference);

  if (result != 0)
    dtdtc_error(ctx, "invalid length [%s]", value);
}

void
svg_x_length_set(dtdtc_context_t ctx, double *attribute,
		 const xmlChar *value, void *data) {
  double *x_base;
  dtdtc_element_t parent;
  int result;

  parent = dtdtc_lookup_parent_by_attribute(ctx, ctx->element, "width");
  if (parent) {
    x_base = dtdtc_attribute_get(ctx, parent, "width");
    result = svg_css_length_set(attribute, value, *x_base);
  } else {
    result = svg_css_length_set(attribute, value, 1.0);
  }

  if (result != 0)
    dtdtc_error(ctx, "invalid width [%s]", value);
}

void
svg_y_length_set(dtdtc_context_t ctx, double *attribute,
		 const xmlChar *value, void *data) {
  svg_player_t player;
  double *y_base;
  dtdtc_element_t parent;
  int result;

  player = (svg_player_t)ctx->user_data;

  parent = dtdtc_lookup_parent_by_attribute(ctx, ctx->element, "height");
  if (parent) {
    y_base = dtdtc_attribute_get(ctx, parent, "height");
    result = svg_css_length_set(attribute, value, *y_base);
  } else {
    result = svg_css_length_set(attribute, value, 1.0);
  }

  if (result != 0)
    dtdtc_error(ctx, "invalid height [%s]", value);
}


void
svg_opacity_set(dtdtc_context_t ctx, double *attribute,
		const xmlChar *value, void *data) {
  /* TODO */
}
