/*
 * $Id: ih-core.h,v 1.1.1.1 2002/03/25 13:07:26 blah Exp $
 */

#ifndef IH_CORE_H
#define IH_CORE_H

#include <sys/time.h>
#include "svg-parser.h"

typedef struct ih_context *ih_context_t;
typedef struct ih_image *ih_image_t;
typedef int ih_boolean_t;

struct ih_context {
  void * userdata; /* user data associated with context */
};

/**
 * ih_handler_t:
 * @new:
 * @destroy:
 * @next:
 * @previous:
 * @seek:
 * @miss:
 * @releasefb:
 * @open:
 * @close:
 * @seekstart:
 * @decodeframe:
 **/
struct ih_handler {
  /* module management */
  ih_boolean_t (*init)();
  void (*exit)();

  /* instance management */
  ih_image_t (*new)(ih_image_t image);
  void (*destroy)(ih_context_t ctx, ih_image_t image);

  /* virtual methods */
  ih_boolean_t (*decodeframe)(ih_image_t image, ih_boolean_t render_p,ih_boolean_t * updated);
  ih_boolean_t (*sync)(ih_image_t ih, struct timeval *now, struct timeval *start, ih_boolean_t render_p, ih_boolean_t *firstsync);
  ih_boolean_t (*seek)(ih_image_t image, int frame, ih_boolean_t render_p);
  ih_boolean_t (*seekstart)(ih_image_t image, ih_boolean_t render_p);
  ih_boolean_t (*miss)(ih_image_t image, int frame);
  ih_boolean_t (*releasefb)(ih_image_t image, int frame);
  ih_boolean_t (*open)(ih_image_t image);
  ih_boolean_t (*close)(ih_image_t image);
  void (*debug)(ih_image_t image);

};

typedef enum {
	IH_RGB = GL_RGB,
	IH_BGR,
	IH_RGBA = GL_RGBA,
	IH_ARGB,
	IH_BGRA,
	IH_YUV32,
	IH_YUV
} ih_format_t;

struct ih_image {
  svg_element_image_t image;	/* associated image element */
  unsigned int width;
  unsigned int height;
  float x_max;			/* ratio image width/buffer width */
  float y_max;			/* ratio image height/buffer height */
  unsigned int alignment;
  void *pixels;
  ih_format_t pixformat;	/* RGB/RGBA/BGRA/BGR/YUV ... */
  int channels;                 /* 3 for rgb, 4 for rgba */
  int frames;                   /* total # of frames */
  int frameno;                  /* current frame number */
  int frameskip;					  /* cumulated number of skipped frames */
  double fps;			/* frames per second */
  double display_fps;           /* display fps */
  unsigned int frameduration;
  struct timeval start;         /* time origin */
  struct ih_handler *handler;
};

/**
 * ih_init:
 *
 * initialize the library.
 *
 * return value: TRUE on success, FALSE otherwise.
 **/
extern ih_boolean_t ih_init(void);
extern void ih_exit(void);

/**
 * ih_context_new:
 * @userdata:
 *
 * creates and initializes a new ih namespace.
 *
 * returns: created context.
 **/
extern ih_context_t ih_context_new(void * userdata);

/**
 * ih_context_destroy:
 * @ctx: context to destroy
 **/
extern void ih_context_destroy(ih_context_t ctx);

/**
 * ih_image_new:
 * @image: a #svg_element_image_t that needs image handler.
 * @name: image URI
 *
 * opens given filename and creates a handler for it.
 *
 * return value: created image handle, NULL on error.
 **/
extern ih_image_t
ih_image_new(svg_element_image_t image, dtdtc_string_t name);

extern void
ih_image_destroy(ih_context_t ctx, ih_image_t img);

/**
 * ih_image_sync:
 * ih:
 * now:
 * start:
 * render_p: whether to render.
 *
 * time seek.
 **/
extern ih_boolean_t ih_image_sync(ih_image_t ih, struct timeval *now, struct timeval *start, ih_boolean_t render_p, ih_boolean_t *firstsync);

extern void ih_image_debug(ih_image_t ih);

extern ih_boolean_t ih_image_releasefb(ih_image_t ih);
extern int ih_image_width(ih_image_t ih);
extern int ih_image_height(ih_image_t ih);
extern int ih_image_alignment(ih_image_t ih);
extern int ih_image_frameno(ih_image_t ih);
extern int ih_image_frames(ih_image_t ih);
extern void *ih_image_pixels(ih_image_t ih);
extern ih_boolean_t ih_image_decodeframe(ih_image_t ih, ih_boolean_t render_p, ih_boolean_t * updated);
extern int ih_image_seekstart(ih_image_t ih, ih_boolean_t render_p);
extern int ih_image_channels(ih_image_t ih);
extern int ih_image_pixformat(ih_image_t ih);
extern float ih_image_x_max(ih_image_t ih);
extern float ih_image_y_max(ih_image_t ih);

extern unsigned int ih_next_pow_2(unsigned int n);

#endif /* IH_CORE_H */
