/*
 *
 */
#include <stdlib.h>
#include <pthread.h>
#include <string.h>
#include <math.h>
#include "ih-divx.h"

/* frame read buffer -- grows on demand */
char *divx_readbuf = NULL;

/* size of the allocated divx_readbuf */
int divx_readbuf_size = 0;

/* instance management */
ih_boolean_t
divx_init(void) {
	divx_readbuf=NULL;

	if (yuvrgb_init(0) != 0) 
		return 0;
	return 1;
}

void
divx_exit(void) {

  if (divx_readbuf)
	  free(divx_readbuf);

  yuvrgb_exit();
}


ih_image_t
divx_new(ih_image_t ih) {
  ih_image_divx_t dh;

  if (!ih) return NULL;

  dh = realloc(ih, sizeof(struct ih_image_divx));

  if (dh) {
    dh->handle = 0;
    dh->surf = NULL;
    dh->avifile = NULL;
    dh->dec_param = NULL;
/*     dh->quality = 0; */
/*     dh->output_format=IH_RGB; */
/*     dh->forced_fps=0; */
    dh->skip_first=0;
    dh->skip_last=0;
  } else {
    free(ih);
  }
  return (ih_image_t)dh;
}

void divx_destroy(ih_context_t ctx, ih_image_t image) {
	divx_close(image);
}

/* methods */

ih_boolean_t
divx_decodeframe(ih_image_t ih, ih_boolean_t render_p, ih_boolean_t *updated) {
  ih_image_divx_t dh = (ih_image_divx_t)ih;
  DEC_FRAME dec_frame;
  DEC_PICTURE dec_pic;
  int key;
  int frame_size;
  int bufsize;
  int error;

  /* grow read buffer if needed */
  frame_size = AVI_frame_size(dh->avifile, ih->frameno);
  if (!frame_size) return 0;

  if (divx_readbuf_size < frame_size) {
    if (divx_readbuf) 
      divx_readbuf = realloc(divx_readbuf, frame_size);
    else
      divx_readbuf = malloc(frame_size);
  }

  bufsize = AVI_read_frame(dh->avifile, divx_readbuf, &key);
  memset(&dec_frame, 0, sizeof(DEC_FRAME));

  dec_frame.bmp=&dec_pic;
  dec_frame.bitstream = divx_readbuf;
  dec_frame.length = bufsize;
  dec_frame.render_flag = render_p;
  dec_frame.stride = ih->width;

  error = decore(dh->handle, DEC_OPT_FRAME, &dec_frame, NULL);
  if (error != DEC_OK) {
    printf("decore(decode) failed [%d]\n", error);
    return 0;
  };
  ih->frameno ++;

  if (render_p) {
    yuvrgb_conv(YUVRGB_YV12_RGB24,
		0,
		dec_pic.y,
		dec_pic.stride_y,
		dec_pic.u,
		dec_pic.v,
		dec_pic.stride_uv,
		dh->surf->pixels,
		dh->surf->w,
		dh->surf->h,
		dec_frame.stride);
    if (updated) *updated=1;
  } else {
    if (updated) *updated=0;
  }
  ih->pixels = dh->surf->pixels;

  return 1;
}

ih_boolean_t
divx_sync(ih_image_t ih, struct timeval *now, struct timeval *start, ih_boolean_t render_p, ih_boolean_t *firstsync) {
  ih_image_divx_t dh = (ih_image_divx_t)ih;

  if (firstsync)
    *firstsync=0;
  if (ih->start.tv_sec == 0) {     /* first play ... */
    /* if first sync, set flag */
    if (firstsync) {
      *firstsync=1;
    }

    memcpy(&ih->start, start, sizeof(struct timeval));
    return ih_image_seekstart(ih, render_p);
  }
  else if (ih->frameno >= ih->frames) { /* ... or last frame */
    ih->start.tv_sec += ((ih->frames - dh->skip_first) *ih->frameduration)/1000000;
    ih->start.tv_usec += (ih->frames - dh->skip_first) *ih->frameduration - 
      (( (ih->frames-dh->skip_first)*ih->frameduration)/1000000)*1000000;
    while (ih->start.tv_usec >=1000000) {
      ih->start.tv_sec++;
      ih->start.tv_usec -= 1000000;
    }

    return ih_image_seekstart(ih, render_p);
  } else {
    int elapsed;
    int newframeno, nbskip;
    int i,updated;

    elapsed = ((now->tv_usec - ih->start.tv_usec)
               + (now->tv_sec - ih->start.tv_sec) * 1000000 );

    if (elapsed<0)
      return 0;

    newframeno = dh->skip_first + (int)floor((double)elapsed / ih->frameduration);

    if (newframeno <= ih->frameno) 
      return 0;

    nbskip = newframeno - ih->frameno - 1;

    /*    if (render_p == 0) {
	  nbskip ++;
	  }
    */

    for (i=0; i < nbskip; i++) {
      if ( !ih_image_decodeframe(ih, 0,NULL) )
	return 0;
      ih->frameskip++;
    }

    ih->display_fps=((double)(newframeno-ih->frameskip))/((double)elapsed/1000000.0);

    /* render current frame, if wanted */
    if (render_p) {
      if ( !ih_image_decodeframe(ih, 1, &updated) ){
        printf("Unable to renderframe\n");
        return 0;
      }
      if (updated)
	return 1;
    }
  }

  return 0;
}


ih_boolean_t
divx_seekstart(ih_image_t image, ih_boolean_t render_p) {
  ih_image_divx_t dh;

  if (!image) return 0;

  dh = (ih_image_divx_t)image;

  AVI_seek_start(dh->avifile);

  image->frameno = 0;
  
  /* skip first frames */
  while ((image->frameno+1) < dh->skip_first) {
     if ( !ih_image_decodeframe(image, 0,NULL) )
       return 0;
  }
  image->frameskip=0;

  return 1;
}

ih_boolean_t
divx_open(ih_image_t ih) {
  DEC_MEM_REQS dec_mem;
  DEC_PARAM * dec_param;
  avi_t *avifile;
  ih_image_divx_t dh = (ih_image_divx_t)ih;

  avifile = AVI_open_input_file(ih->image->xlink_href, 1);

  if (!avifile) return 0;

  if (strcasecmp(AVI_video_compressor(avifile), "divx")
      && strcasecmp(AVI_video_compressor(avifile), "div1")) {
    printf("Not an OpenDIVX format [%s]\n",
	    AVI_video_compressor(avifile));
    return 0;
  };

  dh->avifile = avifile;
  dh->handle = (unsigned int)dh;
  ih->alignment = 4;
  ih->channels = 3;
  ih->pixformat = IH_RGB; 
  ih->frameno = 0;
  ih->frameskip = 0;
  ih->frames = AVI_video_frames(avifile);

  if (dh->skip_first + dh->skip_last < ih->frames) {
    ih->frames -= dh->skip_last;
  } else {
    printf("Invalid frame_interval: total frames:%d skip_first:%d skip_last:%d",ih->frames,dh->skip_first,dh->skip_last);
    dh->skip_first=0;
    dh->skip_last=0;
  }

  ih->fps = AVI_frame_rate(avifile);
  ih->display_fps = ih->fps;
  if (!ih->fps) {
    printf("Invalid frame rate\n");
    return 0;
  }

  ih->frameduration = (unsigned int)(1000000.0/ih->fps);

  dh->dec_param = malloc(sizeof(DEC_PARAM));
  if (!dh->dec_param) return 0;
  dec_param = dh->dec_param;

  memset(dec_param, 0, sizeof(DEC_PARAM));
  dec_param->x_dim = AVI_video_width(avifile);
  dec_param->y_dim = AVI_video_height(avifile);

  ih->width = ih_next_pow_2(dec_param->x_dim);
  ih->height = ih_next_pow_2(dec_param->y_dim);

  ih->x_max = (float)dec_param->x_dim / ih->width;
  ih->y_max = (float)dec_param->y_dim / ih->height;

  dec_param->output_format=DEC_USER;
  dec_param->time_incr=0;

  if ( decore(dh->handle, DEC_OPT_MEMORY_REQS, dec_param, &dec_mem) != DEC_OK )
    return 0;

  dh->surf = SDL_CreateRGBSurface(SDL_SWSURFACE, ih->width, ih->height, 24,
				  0xFF000000,0x00FF0000,0x0000FF00,0x000000FF);
  if (!dh->surf) return 0;

  ih->pixels = dh->surf->pixels;

  dec_param->buffers.mp4_edged_ref_buffers=malloc(dec_mem.mp4_edged_ref_buffers_size);
  dec_param->buffers.mp4_edged_for_buffers=malloc(dec_mem.mp4_edged_for_buffers_size);
  dec_param->buffers.mp4_edged_back_buffers = malloc(dec_mem.mp4_edged_back_buffers_size);
  dec_param->buffers.mp4_display_buffers=malloc(dec_mem.mp4_display_buffers_size);
  dec_param->buffers.mp4_state=malloc(dec_mem.mp4_state_size);
  dec_param->buffers.mp4_tables=malloc(dec_mem.mp4_tables_size);
  dec_param->buffers.mp4_stream=malloc(dec_mem.mp4_stream_size);
  dec_param->buffers.mp4_reference=malloc(dec_mem.mp4_reference_size);

  memset(dec_param->buffers.mp4_state,0,dec_mem.mp4_state_size);
  memset(dec_param->buffers.mp4_tables,0,dec_mem.mp4_tables_size);
  memset(dec_param->buffers.mp4_stream,0,dec_mem.mp4_stream_size);
  memset(dec_param->buffers.mp4_reference,0,dec_mem.mp4_reference_size);

  if ( decore(dh->handle, DEC_OPT_INIT, dec_param, NULL) != DEC_OK ) {
    printf("decore(DEC_OPT_INIT) failed\n");
    return 0;
  };

  return 1;
}

ih_boolean_t
divx_releasefb(ih_image_t ih, int frame) {
  if (!ih) return 0;
  return 1;
}


ih_boolean_t
divx_close(ih_image_t image) {

  if (image) {
    ih_image_divx_t dh = (ih_image_divx_t)image;

    if (dh->surf) {
      SDL_FreeSurface(dh->surf);
      dh->surf=NULL;
    }

    decore(dh->handle, DEC_OPT_RELEASE,NULL,NULL);
    if (dh->dec_param) {
      free(dh->dec_param->buffers.mp4_edged_ref_buffers);
      free(dh->dec_param->buffers.mp4_edged_for_buffers);
      free(dh->dec_param->buffers.mp4_edged_back_buffers);
      free(dh->dec_param->buffers.mp4_display_buffers);
      free(dh->dec_param->buffers.mp4_state);
      free(dh->dec_param->buffers.mp4_tables);
      free(dh->dec_param->buffers.mp4_stream);
      free(dh->dec_param->buffers.mp4_reference);
      free(dh->dec_param);
      dh->dec_param=NULL;
    }

    if (dh->avifile) {
      AVI_close(dh->avifile);
      dh->avifile=NULL;
    }
  } else return 0;

  return 1;
}

void
divx_debug(ih_image_t ih) {
  ih_image_divx_t dh;

  dh = (ih_image_divx_t)ih;

  printf("   divx: handle=%u, surf=%p, avifile=%p",
	 dh->handle, dh->surf, dh->avifile);
}

