/*
 *
 */

#include "ih-pixbuf.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>

/* instance management */

ih_image_t pixbuf_new(ih_image_t ih) {
  ih_image_pixbuf_t ph;

  ph = realloc(ih, sizeof(struct ih_image_pixbuf));

  if (ph) {
    ph->surf = NULL;
  }
  else
    free(ih);

  return (ih_image_t)ph;
}

void pixbuf_destroy(ih_context_t ctx, ih_image_t ih) {
  pixbuf_close(ih);
}

/* methods */

ih_boolean_t
pixbuf_decodeframe(ih_image_t ih, ih_boolean_t render_p,ih_boolean_t *updated) {
  ih_image_pixbuf_t ph = (ih_image_pixbuf_t)ih;

  if (!ph->surf) {/* fb not allocated or open not done */
    ih->pixels=NULL;
    *updated=0;
    return 1;
  }

  ih->pixels = ph->surf->pixels;
  ih->frameno++; /* increment frameno to detect "end" of image */

  if (updated)
    *updated=0;

  return 1;
}

ih_boolean_t
pixbuf_seekstart(ih_image_t ih, ih_boolean_t render_p) {
  ih->frameno = 0;
  ih->frameskip=0;

  return 1;
}

ih_boolean_t
pixbuf_open(ih_image_t ih) {
  ih_image_pixbuf_t ph = (ih_image_pixbuf_t)ih;
  SDL_Surface *surf;

  surf = IMG_Load(ih->image->xlink_href);

  if (!surf) return 0;

  ih->width = ih_next_pow_2(surf->w);
  ih->height = ih_next_pow_2(surf->w);
  if (ih->width != surf->w || ih->height != surf->h) {
    /* needs reblit */
    SDL_Rect area = {0, 0, surf->w, surf->h};
    ph->surf = SDL_CreateRGBSurface(SDL_SWSURFACE, ih->width, ih->height,
				    surf->format->BytesPerPixel==3 ? 24 : 32,
				    0xFF000000,
				    0x00FF0000,
				    0x0000FF00,
				    0x000000FF);
    SDL_BlitSurface(surf, &area , ph->surf, &area);
    SDL_FreeSurface(surf);
    ih->x_max = (float)area.w / ih->width;
    ih->y_max = (float)area.h / ih->height;
  } else {
    ph->surf = surf;
    ih->x_max = 1.0;
    ih->y_max = 1.0;
  }

  ih->alignment = 4;
  ih->pixels = ph->surf->pixels;
  ih->channels = ph->surf->format->BytesPerPixel;
  ih->pixformat = (ph->surf->format->BytesPerPixel==3)?IH_RGB:IH_RGBA;
  ih->frameno = 0;
  ih->frameskip = 0;
  ih->frames = 1;
  ih->fps = 1.0;
  ih->display_fps = 1.0;
  if (!ih->fps) return 0;
  ih->frameduration = 1000000/ih->fps;

/*   printf("opening [%s] chans=%d",ih->image->xlink_href, ih->channels); */

  return 1;
}

ih_boolean_t
pixbuf_sync(ih_image_t ih, struct timeval *now, struct timeval *start, ih_boolean_t render_p, ih_boolean_t *firstsync) {

  if (firstsync)
    *firstsync=0;

  if (ih->start.tv_sec == 0) {     /* first play ... */
    /* if first sync, set flag */
    if (firstsync) {
      *firstsync=1;
    }

    memcpy(&ih->start, start, sizeof(struct timeval));
    return ih_image_seekstart(ih, render_p);
  }
  else if (ih->frameno >= ih->frames) { /* ... or last frame */
    ih->start.tv_sec += (ih->frames*ih->frameduration)/1000000;
    ih->start.tv_usec += ih->frames*ih->frameduration - ((ih->frames*ih->frameduration)/1000000)*1000000;
    while (ih->start.tv_usec >=1000000) {
      ih->start.tv_sec++;
      ih->start.tv_usec -= 1000000;
    }

    return ih_image_seekstart(ih, render_p);
  } else {
    int elapsed;
    int newframeno, nbskip;
    int i,updated;

    elapsed = ((now->tv_usec - ih->start.tv_usec)
               + (now->tv_sec - ih->start.tv_sec) * 1000000 );

    if (elapsed<0)
      return 0;

    newframeno = (int)floor((double)elapsed / ih->frameduration);

    if (newframeno <= ih->frameno) 
      return 0;

    nbskip = newframeno - ih->frameno - 1;

    /*    if (render_p == 0) {
	  nbskip ++;
	  }
    */

    for (i=0; i < nbskip; i++) {
      if ( !ih_image_decodeframe(ih, 0,NULL) ) return 0;
      ih->frameskip++;
    }

    ih->display_fps=((double)newframeno-ih->frameskip) / (elapsed/1000000.0);

    /* render current frame, if wanted */
    if (render_p) {
      if ( !ih_image_decodeframe(ih, 1, &updated) ){
        printf("Unable to renderframe\n");
        return 0;
      }
      if (updated)
	return 1;
    }
  }

  return 0;
}


ih_boolean_t pixbuf_releasefb(ih_image_t ih, int frame)
{
  ih_image_pixbuf_t ph = (ih_image_pixbuf_t)ih;

  if (ph->surf) {
    SDL_FreeSurface(ph->surf);
    ph->surf=NULL;
    ih->pixels=NULL;
  }

  return 1;
}

ih_boolean_t
pixbuf_close(ih_image_t ih) {
  pixbuf_releasefb(ih,0);
  return 1;
}

void
pixbuf_debug(ih_image_t ih) {
  ih_image_pixbuf_t ph;

  ph = (ih_image_pixbuf_t)ih;

}
