#include <GL/glu.h>
#include "svg-parser.h"
#include "player.h"

svg_player_t
svg_player_create() {
  svg_player_t player;

  if ( !ih_init() ) return NULL;

  svg_text_init();

  player = malloc( sizeof(struct svg_player) );
  if (!player) return NULL;

  player->canvas = svg_canvas_create(640, 480);

  return player;
}

static void
player_draw(dtdtc_context_t ctx, svg_element_svg_t svg) {
  svg_player_t player;
  int i;
  GLuint tex_size;

  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  glOrtho( 0.0, 1.0, -1.0, 0.0, 0.0, 1.0 );

  glMatrixMode( GL_MODELVIEW );

  glEnable(GL_LINE_SMOOTH);

  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER );

  glEnable(GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glGetIntegerv(GL_MAX_TEXTURE_SIZE,&tex_size);
  printf("max texture size = %d\n", tex_size);
  
  player = (svg_player_t)ctx->user_data;
  i = 20;
  while (i) {
/*     i++; */
    glClearColor( 0.5, 0.5, 0.5, 1.0 );
    glClear( GL_COLOR_BUFFER_BIT);
    /* draw root node */
    svg_draw_element((dtdtc_element_t)svg);

    /* render canvas */
    svg_canvas_draw(player->canvas);
/*     sleep(1); */
  }

}

static dtdtc_ns_t context_namespaces[] = {
  &xlink_ns, &svg_ns, 0
};

int
main(int argc, char **argv) {
  dtdtc_context_t ctx;
  svg_player_t player;
  svg_element_svg_t svg;

  player = svg_player_create();
  if (!player) return 1;

  ctx = dtdtc_context_init(context_namespaces, (void*)player);

  svg = (svg_element_svg_t)dtdtc_context_parse_file(ctx, argv[1], NULL);
  
  if (!svg) {
    printf("invalid root element\n");
    return -1;
  }

  if (svg->element.type != SVG_ELEMENT_SVG) {
    printf("invalid root element type: %d\n", svg->element.type);
    return -2;
  }

  player_draw(ctx, svg);

  dtdtc_context_exit(ctx);

  ih_exit();
  return 0;
}
