#include <string.h>
#include <math.h>
#include "svg-parser.h"
#include "player.h"

int
svg_text_init() {
  char **fontpath;
  char *envstr, *str;
  char *p, *p2;

  int nbpaths;
  int ret;

  envstr=getenv("FONTPATH");
  if (envstr && strlen(envstr)>0 ) 
    str=strdup(envstr);
  else 
    str=strdup("/usr/lib/X11/fonts/TrueType");

  if (!str) return 0;

  p=str;
  nbpaths=0;
  fontpath=NULL;
  do {
    p2=strchr(p,':');
    if (p2)  
      *p2='\0';

    /* path is p */
    fontpath = realloc(fontpath, (nbpaths+2)*sizeof(char *));
    if (!fontpath) {
      free(str);
      return 0;
    }
    fontpath[nbpaths] = p;
    fontpath[nbpaths+1] = NULL;
    nbpaths++;

    p = p2 + 1;
  } while (p2);

  ret=IFT_Init(fontpath);

  free(fontpath);
  free(str);

  if (ret)
    return 0;

  return 1;
}

void
svg_text_end(dtdtc_context_t ctx) {
  svg_element_text_t text;
  dtdtc_string_t font_family;
  dtdtc_element_t parent;
  double *x_base, *y_base, reference;

  text = (struct svg_element_text*)ctx->element;

  if (!text->font_family)
    font_family = svg_font_font_family_inherit(ctx, ctx->element);
  else 
    font_family = text->font_family;

  if (!font_family) return;

  switch (text->font_style) {
  case SVG_ENUM_USE_FONT_STYLE_INHERIT:
  case SVG_ENUM_USE_FONT_STYLE_ITALIC:
  case SVG_ENUM_USE_FONT_STYLE_OBLIQUE:
  case SVG_ENUM_USE_FONT_STYLE_NORMAL:
  default:
    text->ift = IFT_New(font_family, "Regular");
  }
  if (!text->ift) {
    dtdtc_warning(ctx, "can't create font [%s-%s-%d]",
		  font_family, text->font_style, text->font_size);
    return;
  }

  IFT_SetFgColor(text->ift,
		 text->fill.color.r, text->fill.color.g,
		 text->fill.color.b, text->fill.color.a);
  IFT_SetBgColor(text->ift, 0.0, 0.0, 0.0, 1.0);

  IFT_SetSize(text->ift, text->font_size, 0);
  IFT_SetText(text->ift, text->element.content);

  if ( IFT_Render(text->ift) ) {
    dtdtc_warning(ctx, "failed to render text [%s]",
		  text->element.content);
  }

  /* compute sizes */
  parent = dtdtc_lookup_parent_by_attribute(ctx, ctx->element, "width");
  x_base = dtdtc_attribute_get(ctx, parent, "width");
  y_base = dtdtc_attribute_get(ctx, parent, "height");
  reference = sqrt(*x_base * *x_base + *y_base * *y_base) / sqrt(2);

  text->width = text->ift->pixbuf->w / reference;
  text->height = text->ift->pixbuf->h / reference;

  /* text is rendered in a 300dpi device (see fontTT.c), correct values here */
/*   text->width *= 75.0 / 300; */
/*   text->height *= 75.0 / 300; */

/*   glGenTextures(1, &text->texId); */
/*   glBindTexture(GL_TEXTURE_2D, text->texId); */

}

void
svg_text_draw(struct svg_element_text *text) {
  GLuint pixformat;

/*   glBindTexture(GL_TEXTURE_2D, text->texId); */
  glDisable(GL_TEXTURE_2D);

/*   glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR ); */
/*   glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR ); */
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glPixelStorei(GL_UNPACK_ROW_LENGTH, text->ift->pixbuf->aw);
  glPixelStoref(GL_UNPACK_SKIP_PIXELS, 0);
  glPixelStoref(GL_UNPACK_SKIP_ROWS, 0);
  glRasterPos2f(text->x, -text->y);

  pixformat =  text->ift->pixbuf->nchans == 3 ? GL_RGB : GL_RGBA; 

  glDrawPixels(text->ift->pixbuf->w,
	       text->ift->pixbuf->h,
	       pixformat,
	       GL_UNSIGNED_BYTE,
	       text->ift->pixbuf->pixels);

#if 0
  glTexImage2D(GL_TEXTURE_2D, 0, pixformat, 
	       text->ift->pixbuf->aw,
	       text->ift->pixbuf->ah,
	       0,
	       pixformat,
	       GL_UNSIGNED_BYTE,
	       text->ift->pixbuf->pixels);


  printf("printing [%s] @(%f,%f) (%fx%f) (%dx%dx%d)\n",
	 text->element.content, text->x, - text->y,
	 text->width, text->height,
	 text->ift->pixbuf->w,
	 text->ift->pixbuf->h,
	 text->ift->pixbuf->nchans);

  glPushMatrix();
  glTranslatef(text->x + text->width / 2,
	       -text->y - text->height / 2,
	       0.0);
  glScalef(text->width, text->height, 1.0);
  glColor4f(1.0, 1.0, 1.0, 1.0);

  {
    glBegin(GL_QUADS);

    glTexCoord2f(0.0, 0.0);
    glVertex2f(-0.5, 0.5);

    glTexCoord2f((float)text->ift->pixbuf->w / text->ift->pixbuf->aw, 0.0);
    glVertex2f(0.5, 0.5);

    glTexCoord2f((float)text->ift->pixbuf->w / text->ift->pixbuf->aw,
		 (float)text->ift->pixbuf->h / text->ift->pixbuf->ah);
    glVertex2f(0.5, -0.5);

    glTexCoord2f(0.0, (float)text->ift->pixbuf->h / text->ift->pixbuf->ah);
    glVertex2f(-0.5, -0.5);

    glEnd();
  }

  glPopMatrix();
#endif

}

void
svg_text_destroy(dtdtc_context_t ctx) {
  svg_element_text_t text;

  text = (struct svg_element_text*)ctx->element;
  IFT_Free(text->ift);
/*   glDeleteTextures(1, &text->texId); */
}

